# Generated by Django 3.2.5 on 2023-02-02 08:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PlatformsToUsers',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('platform_user_id', models.BigIntegerField(blank=True, db_index=True, null=True, verbose_name='平台用户ID')),
            ],
            options={
                'verbose_name_plural': '09. 用户 - 多对多平台记录表',
                'db_table': 'user_platforms_to_users',
                'managed': False,
            },
        ),
    ]



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 返回协议封装工具
@created_time: 2022/6/15 14:14
"""

from django.http import JsonResponse
from rest_framework import status


# 数据返回规则
def util_response(data='', err=0, msg='ok', tip=0, http_status=status.HTTP_200_OK):
    """
    http 返回协议封装
    :param data: 返回的数据体
    :param err: 错误码，一般以1000开始，逐一增加。登录错误为6000-6999。
    :param msg: 错误信息，一般为服务返回协议中的err
    :param tip: 提示信息，当err为0时候需要前端提示则使用该字段传递提示信息。
    :param http_status: 返回状态码
    :return: response对象
    """
    if http_status == status.HTTP_200_OK:
        return JsonResponse({'err': err, 'msg': msg, 'data': data, "tip": tip})
    else:
        return JsonResponse({'err': http_status, 'msg': msg, "tip": tip}, status=http_status)



from django.contrib import admin

from config.config import JConfig
# 引入用户平台
from .models import *

config = JConfig()


# Register your models here.


class BaseInfoAdmin(admin.ModelAdmin):
    # fields = ('user_name', 'full_name', 'platform', 'platform_uid', 'phone', 'email', 'wechat', 'user_info')
    fields = (
    'id', 'user_name', 'full_name', 'nickname', 'phone', 'email', 'wechat_openid', 'user_info', 'register_time')
    list_display = (
        'id', 'user_name', 'full_name', 'nickname', 'phone', 'email', 'wechat_openid', 'user_info', 'register_time')
    search_fields = ('user_name', 'full_name', 'nickname', 'phone',)
    list_filter = []
    readonly_fields = ['id']


class AuthAdmin(admin.ModelAdmin):
    fields = ('id', 'user', 'password', 'salt', 'algorithm', 'token', 'ticket', 'create_time', 'update_time')
    list_display = ('user', 'password', 'token', 'create_time', 'update_time',)
    search_fields = ('user__user_name', 'user__full_name', 'create_time')
    list_filter = ['user']
    readonly_fields = ['id', 'create_time', 'update_time']

    # def platform(self, obj):
    #     return obj.platform


class DetailInfoAdmin(admin.ModelAdmin):
    fields = (
        'id', 'user', 'real_name', 'sex', 'birth', 'tags', 'signature', 'avatar', 'cover', 'language', 'region_code',
        'more',
        'field_1', 'field_2', 'field_3', 'field_4', 'field_5', 'field_6', 'field_7', 'field_8', 'field_9',
        'field_10', 'field_11', 'field_12', 'field_13', 'field_14', 'field_15'
    )
    list_display = ('user', 'real_name', 'sex', 'birth', 'tags', 'avatar', 'cover', 'language', 'region_code')
    readonly_fields = ['id']


class ExtendFieldAdmin(admin.ModelAdmin):
    fields = ('id', 'field', 'field_index', 'description', 'type', 'config', 'default', 'sort')
    list_display = ('field', 'field_index', 'description', 'type', 'config', 'default', 'sort')
    readonly_fields = ['id']


class AccessLogAdmin(admin.ModelAdmin):
    fields = ('id', 'user', 'ip', 'create_time', 'client_info', 'more',)
    list_display = ('user', 'ip', 'create_time', 'client_info',)
    readonly_fields = ['id', 'create_time']


class HistoryAdmin(admin.ModelAdmin):
    fields = ('id', 'user', 'field', 'old_value', 'new_value', 'create_time',)
    list_display = ('user', 'field', 'old_value', 'new_value', 'create_time',)
    readonly_fields = ['id', 'create_time']


class RestrictRegionAdmin(admin.ModelAdmin):
    fields = ('id', 'user', 'region_code',)
    list_display = ('user', 'region_code',)
    readonly_fields = ['id']


class PlatformAdmin(admin.ModelAdmin):
    fields = ('platform_id', 'platform_name')
    list_display = ('platform_id', 'platform_name')
    search_fields = ('platform_id', 'platform_name')


class PlatformsToUsersAdmin(admin.ModelAdmin):
    fields = ('id','user', 'platform', 'platform_user_id',)
    list_display = ('platform','user', 'platform_user_id',)
    readonly_fields = ['id']


# class PermissionAdmin(admin.ModelAdmin):
#     fields = ('permission_id', 'permission_name',)
#     list_display = ('permission_id', 'permission_name',)


# class PermissionValueAdmin(admin.ModelAdmin):
#     fields = ('id', 'permission', 'module', 'feature', 'permission_value', 'type',
#               'relate_value', 'config', 'is_enable', 'is_system', 'is_ban', 'ban_view', 'ban_edit', 'ban_add',
#               'ban_delete',
#               'description')
#     list_display = ('permission', 'permission_value', 'type', 'is_system', 'is_ban',)
#     readonly_fields = ['id']


# class GroupAdmin(admin.ModelAdmin):
#     fields = ('id', 'group', 'parent_group', 'description')
#     list_display = ('id', 'group', 'parent_group', 'description')
#     readonly_fields = ['id']


class ContactBookAdmin(admin.ModelAdmin):
    fields = ('id', 'user_id', 'friend', 'phone', 'phones', 'telephone', 'telephones',
              'email', 'qq', 'address', 'more', 'remarks')
    list_display = ('id', 'user_id', 'friend', 'phone', 'phones', 'telephone', 'telephones',
                    'email', 'qq', 'address', 'more', 'remarks')
    readonly_fields = ['id']


class UserSsoServeAdmin(admin.ModelAdmin):
    fields = ('id', 'sso_code', 'sso_name', 'sso_url', 'description')
    list_display = ('id', 'sso_code', 'sso_name', 'sso_url', 'description')
    readonly_fields = ['id']


class UserSsoToUserAdmin(admin.ModelAdmin):
    fields = ('id', 'sso_serve', 'user', 'sso_unicode', 'sso_ticket', 'app_id')
    list_display = ('id', 'sso_serve', 'user', 'sso_unicode', 'sso_ticket', 'app_id')
    readonly_fields = ['id']


class UserRelateTypeAdmin(admin.ModelAdmin):
    fields = ("id", "relate_key", "relate_name", "description",)
    list_display = ("id", "relate_key", "relate_name", "description",)
    readonly_fields = ['id']


class UserRelateToUserAdmin(admin.ModelAdmin):
    fields = ("id", "user", "with_user", "user_relate_type",)
    list_display = ("id", "user", "with_user", "user_relate_type",)
    readonly_fields = ['id']


admin.site.register(BaseInfo, BaseInfoAdmin)
admin.site.register(Auth, AuthAdmin)
admin.site.register(DetailInfo, DetailInfoAdmin)
admin.site.register(AccessLog, AccessLogAdmin)
admin.site.register(History, HistoryAdmin)
admin.site.register(RestrictRegion, RestrictRegionAdmin)
admin.site.register(Platform, PlatformAdmin)
admin.site.register(PlatformsToUsers, PlatformsToUsersAdmin)
# admin.site.register(Permission, PermissionAdmin)
# admin.site.register(PermissionValue, PermissionValueAdmin)
# admin.site.register(Group, GroupAdmin)
admin.site.register(ContactBook, ContactBookAdmin)
admin.site.register(ExtendField, ExtendFieldAdmin)
admin.site.register(UserSsoServe, UserSsoServeAdmin)
admin.site.register(UserSsoToUser, UserSsoToUserAdmin)
admin.site.register(UserRelateType, UserRelateTypeAdmin)
admin.site.register(UserRelateToUser, UserRelateToUserAdmin)



# _*_coding:utf-8_*_

from datetime import datetime, timedelta
import re

from django.contrib.auth.hashers import check_password
from django.db.models import Q
import jwt
from rest_framework import serializers

from config.config import Config
from ..models import Auth
from ..models import BaseInfo


class UserInfoSerializer(serializers.ModelSerializer):
    # 方法一：使用SerializerMethodField，并写出get_platform, 让其返回你要显示的对象就行了
    # p.s.SerializerMethodField在model字段显示中很有用。
    # platform = serializers.SerializerMethodField()

    # # 方法二：增加一个序列化的字段platform_name用来专门显示品牌的name。当前前端的表格columns里对应的’platform’列要改成’platform_name’
    user_id = serializers.ReadOnlyField(source='id')

    # platform_id = serializers.ReadOnlyField(source='platform.platform_id')
    # platform_name = serializers.ReadOnlyField(source='platform.platform_name')

    class Meta:
        model = BaseInfo
        fields = [
            'user_id',
            # 'platform',
            # 'platform_uid',
            # 'platform__platform_name',
            # 'platform_id',
            # 'platform_name',
            'user_name',
            'full_name',
            'phone',
            'email',
            'wechat',
            'user_info',
        ]
        # exclude = ['platform_uid']

    # # 这里是调用了platform这个字段拼成了get_platform
    # def get_platform(self, obj):
    #     return obj.platform.platform_name
    #     # return {
    #     #     'id': obj.platform.platform_id,
    #     #     'name': obj.platform.platform_name,
    #     # }


class UserService:
    def __init__(self):
        pass

    # 检测账户
    @staticmethod
    def check_account(account):
        """
        @param account 用户账户，可以支持三种类型：手机、用户名、邮箱。自动判断
        @description 注意：用户名不推荐由纯数字构成，因为那样容易和11位手机号冲突
        """
        # 账号类型判断
        if re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', account):
            account_type = 'phone'
        elif re.match(r'^\w+[\w\.\-\_]*@\w+[\.\w]*\.\w{2,}$', account):
            account_type = 'email'
        elif re.match(r'^[A-z\u4E00-\u9FA5]+\w*$', account):
            account_type = 'username'
        else:
            return {'err': 6010, 'msg': "账号必须是用户名、手机或者邮箱，用户名不能是数字开头", }

        # 用户ID
        user_list = BaseInfo.objects.filter(Q(user_name=account) | Q(phone=account) | Q(email=account))
        if not user_list.count():
            return {'err': 6020, 'msg': "账户不存在", }
        if user_list.count() > 1:
            return {'err': 6030, 'msg': "登录异常，请联系管理员，发现多账号冲突：" + account, }
        # print("> user_list:", user_list)
        user_set = user_list.first()

        serializer = UserInfoSerializer(user_set, many=False)
        # print("> serializer:", serializer)
        return {'err': 0, 'msg': "OK" + account, 'data': serializer.data, }

    # 验证密码
    @staticmethod
    def check_login(user_id, password, account):
        """
        @param user_id 用户ID
        @param password 用户密码。
        @param account 登陆账号，必填，用于生成Token令牌。
        @description 注意：目前密码是明文传输，今后都要改成密文传输
        """
        auth_set = Auth.objects.filter(user_id=user_id, password__isnull=False).order_by('-update_time').first()
        if not auth_set:
            return {'err': 6040, 'msg': "账户尚未开通登录服务：" + account + "(" + user_id + ")", }

        # 判断密码不正确
        is_pass = check_password(password, auth_set.password)
        if not is_pass:
            return {'err': 6050, 'msg': "密码错误：", }

        # 过期时间
        expire_timestamp = datetime.utcnow() + timedelta(days=Config.getIns().get('xj_user', 'DAY', 7), seconds=Config.getIns().get('xj_user', 'SECOND', 0))
        # 为本次登录生成Token并记录
        # todo 漏洞，当用户修改用户名时，并可能导致account失效，是否存用户ID更好
        token = jwt.encode(payload={'account': account, 'user_id': user_id, "exp": expire_timestamp}, key=Config.getIns().get("xj_user", 'JWT_SECRET_KEY'))
        payload = jwt.decode(token, key=Config.getIns().get("xj_user", 'JWT_SECRET_KEY'), verify=True, algorithms=["RS256", "HS256"])
        print("> payload:", payload)
        auth_set.token = token
        auth_set.save()

        return {'err': 0, 'msg': "OK：", 'data': {'token': token}, }

    # 检测令牌
    @staticmethod
    def check_token(token):
        """
        @param token 用户令牌。
        @description 注意：用户令牌的载荷体payload中必须包含两个参数：account账号、exp过期时间，其中账号可以是手机、用户名、邮箱三种。
        @description BEARER类型的token是在RFC6750中定义的一种token类型，OAuth2.0协议RFC6749对其也有所提及，算是对RFC6749的一个补充。BEARER类型token是建立在HTTP/1.1版本之上的token类型，需要TLS（Transport Layer Security）提供安全支持，该协议主要规定了BEARER类型token的客户端请求和服务端验证的具体细节。
        @description 理论上，每次请求令牌后就更新一次令牌，以监测用户长期访问时，不至于到时间后掉线反复登陆。
        """
        # 检查是否有Bearer前辍，如有则截取
        print("> token 1:", token)
        if re.match(r'Bearer (.*)', token, re.IGNORECASE):
            token = re.match(r'Bearer (.*)', token, re.IGNORECASE).group(1)
        # print("> token 2:", token)

        # # 验证token。另一种方式，从数据库核对Token，通过对比服务端的Token，以确定是否为服务器发送的。今后启用该功能。
        # auth_set = Auth.objects.filter(Q(token=token)).order_by('-update_time')
        # # print("> auth:", auth_set)
        # if not auth_set.count():
        #     raise MyApiError('token验证失败', 6002)
        # auth = auth_set.first()

        try:
            # jwt.decode会自动检查exp参数，如果超时则抛出jwt.ExpiredSignatureError超时
            jwt_payload = jwt.decode(token, key=config.JWT_AUTH['JWT_SECRET_KEY'], verify=True, algorithms=["RS256", "HS256"])
            print("> jwt_payload:", jwt_payload)

        except jwt.ExpiredSignatureError:
            # raise exceptions.AuthenticationFailed('登录已过期，请重新登录！')
            return {'err': 6060, 'msg': "登录已过期，请重新登录", }

        except jwt.InvalidTokenError:
            # raise exceptions.AuthenticationFailed('用户令牌无效，请重新登录！')
            return {'err': 6070, 'msg': "用户令牌无效，请重新登录", }

        account = jwt_payload.get('account', None)
        user_id = jwt_payload.get('user_id', None)
        if not account:
            return {'err': 6080, 'msg': "错误：令牌载荷中未提供用户账户Account", 'data': {'payload': jwt_payload}, }

        # 检测用户令牌时不应该调用用户信息，这会导致任何接口都会查询用户表，时间会增加
        # user_info = UserService.check_account(account=account)

        return {'err': 0, 'msg': "OK", 'data': {'account': account, 'user_id': user_id}, }

        # return {'account': account}, "OK", 0



# _*_coding:utf-8_*_

from rest_framework.views import APIView

from xj_role.services.role_service import RoleService
from xj_role.services.user_group_service import UserGroupService
from ..services.user_service import UserService
from ..utils.custom_tool import request_params_wrapper
from ..utils.model_handle import util_response
from ..utils.user_wrapper import user_authentication_force_wrapper


# 管理员添加用户
class UserAdd(APIView):

    @user_authentication_force_wrapper
    @request_params_wrapper
    def post(self, request, *args, request_params=None, **kwargs):
        if request_params is None:
            request_params = {}

        # 获取角色和部门的id列表
        user_role_list = request_params.pop('user_role_list', None)
        user_group_list = request_params.pop('user_group_list', None)

        # 进行用户添加
        data, err = UserService.user_add(request_params)
        if err:
            return util_response(err=1001, msg=err)

        # 绑定用户的角色和组织
        if user_group_list:
            UserGroupService.bind_user_group(data.get("user_id"), user_group_list)
        if user_role_list:
            RoleService.bind_user_role(data.get("user_id"), user_role_list)

        return util_response(data=data)
